"""
    CLASS API LOADER to S3 Directory
    https://www.amocrm.com/developers/content/api_v4/leads-2/
"""

import logging
from abc import ABC

from amocrm.api.base.api_loader_base import AmocrmApiLoaderBase

PAGE_NUMBER_MAX = 1e6


def get_limit_from_url(objects):
    """
    get page limit form response
    :param objects:
    :return:
    """
    import urllib.parse as urlparse

    try:
        url = objects.json()["_links"]["self"]["href"]
        par = urlparse.parse_qs(urlparse.urlparse(url).query)
        return int(par["limit"][0])
    except KeyError:
        return 50


def process_json(data, entity):
    """
    update data after retrieving from s3
    Args:
        data: row data json
    Returns: list
    :param data:
    :param entity:
    """
    return data["_embedded"][entity]


class AmocrmApiLoader(AmocrmApiLoaderBase, ABC):
    """
    The main class
    """

    def __init__(
            self,
            entity,
            s3_client,
            args_api,
            date_modified_from=None,
            batch_api=250,
            is_oauth2=True,
    ):
        """
        :param entity:   amocrm entities contacts/users/accounts e.t.c
        :param s3_client: s3 client from talenttech-oss library
        :param args_api: dict with
        :param date_modified_from:
        :param batch_api: size of batch to upload
        """

        AmocrmApiLoaderBase.__init__(
            self,
            entity=entity,
            s3_client=s3_client,
            args_api=args_api,
            date_modified_from=date_modified_from,
            batch_api=batch_api,
            is_oauth2=is_oauth2,
        )

        self.logger = logging.getLogger(__class__.__name__)

    def extract(self):
        """load table from amocrm.api"""
        self.auth()
        self.create_s3_folder()
        self.clear_s3_folder()  # clear old before loading

        url_bases = self.args_api["amocrm_api_url"].split(";")
        num_url = 0
        for url_base in url_bases:
            cur_page = 1

            params = {}
            if self.date_modified_from is not None:
                self.logger.info("Uploading data with after %s ", self.date_modified_from)
                if url_base.find('only_deleted') == -1:
                    params["filter[updated_at][from]"] = int(
                        self.date_modified_from.timestamp()
                    )
            has_next = True
            while has_next and cur_page < PAGE_NUMBER_MAX:
                self.logger.info("Extracting page №%d", cur_page)

                file_path = self.get_file_name(cur_page, self.batch_api, str(num_url))

                url = url_base.format(limit=self.batch_api, page=cur_page)

                try:
                    objects = self.oath_client.get_response_objects(url, params)
                except:
                    import time
                    time.sleep(60)
                    objects = self.oath_client.get_response_objects(url, params)

                self.logger.info(objects.url)
                if objects.status_code != 200:
                    self.logger.warning(
                        "Warning! API sent no result %d,\n %s", objects.status_code, str(objects.content)
                    )
                    break

                limit_from_url = get_limit_from_url(objects)
                if "next" not in objects.json()["_links"]:
                    has_next = False

                if limit_from_url != self.batch_api:
                    logging.warning(
                        "Batch API: %d is too large, changing to default value %d",
                        self.batch_api,
                        limit_from_url,
                    )
                    self.batch_api = limit_from_url

                count_uploaded = len(process_json(objects.json(), self.entity))
                self.rows_to_upload += count_uploaded
                self.logger.info(
                    "Saving data to the file %s, a number of rows %d, total number: %d",
                    file_path,
                    count_uploaded,
                    self.rows_to_upload,
                )
                self.s3_client.create_file(file_path, objects.content)
                if count_uploaded < self.batch_api:
                    logging.warning(
                        "A number of rows %d in response less then size of batch %d. Make a return",
                        count_uploaded,
                        self.batch_api,
                    )
                    break

                cur_page += 1
            self.logger.info(
                "Total number of rows received from API is %d", self.rows_to_upload
            )
            num_url += 1



import logging
import sys

from amocrm.api.auth.oauth_client import AmoOAuthClient


class AmocrmApiLoaderBase:
    """
    The base class for API loader
    """

    def __init__(
            self,
            entity,
            s3_client,
            args_api,
            date_modified_from=None,
            batch_api=250,
            is_oauth2=True,
    ):
        """
        :param entity:   amocrm entities contacts/users/accounts e.t.c
        :param s3_client: s3 client from talenttech-oss library
        :param args_api: dict with AMO_USER_LOGIN/AMO_USER_HASH/AMO_AUTH_URL
        :param date_modified_from:
        :param  batch_api: size of batch to upload
        :param: is_oauth2 if new system authorization
        """
        log_format = "%(asctime)-15s %(name)s:%(levelname)s: %(message)s"
        logging.basicConfig(format=log_format, stream=sys.stdout, level=logging.INFO)
        logging.basicConfig(format=log_format, stream=sys.stderr, level=logging.ERROR)
        logging.captureWarnings(True)
        self.logger = logging.getLogger(__class__.__name__)

        self.entity = entity

        self.args_api = args_api
        self.batch_api = batch_api
        self.date_modified_from = date_modified_from

        self.s3_client = s3_client

        self.rows_to_upload = 0

        self.is_auth2 = is_oauth2
        self.oath_client = None

    def auth(self, code_auth=None):
        """API authorization auth2"""
        self.oath_client = AmoOAuthClient(self.args_api, self.s3_client, self.logger, self.is_auth2)
        self.oath_client.auth(code_auth)

    def create_s3_folder(self):
        if not self.s3_client.path_exists(self.s3_client.root_dir):
            self.s3_client.create_dir(self.s3_client.root_dir)

    def clear_s3_folder(self):
        """Clear all data from s3 folder"""
        if self.s3_client.path_exists(self.s3_client.root_dir):
            for file in self.s3_client.get_file_list(self.s3_client.root_dir):
                self.logger.info("Delete %s from s3", file)
                self.s3_client.delete_file(path=file)
            self.s3_client.delete_dir(self.s3_client.root_dir)  # remove directory

    def get_file_name(self, offset, batch, add_arg=None):
        """
        Returns: s3 file name
        """
        return "{dir_path}/{entity}_{offset}_{batch}_{add_arg}.json".format(
            dir_path=self.s3_client.root_dir,
            entity=self.entity,
            offset=offset,
            batch=batch,
            add_arg=add_arg
        )

    def extract(self):
        """load table from amocrm.api"""
        pass



"""
Module for downloading data from s3 to vertica
"""

import logging
import json

from vconnector.vertica_connector import VerticaConnector
from amocrm.db.base.uploader import UploaderDBBase, get_date_str
from abc import ABC


class UploaderDB(UploaderDBBase, ABC):
    """
    Class to put the data from s3 into vertica
    """

    def __init__(
            self,
            s3_client,
            sql_credentials,
            entity,
            table_name,
            files_in_upload=5,
            full_copy=False,
            json_columns=None,
            api_version=4,
    ):
        """
        :param s3_client: s3 client from talenttech-oss library
        :param sql_credentials: vertica variable dict
        :param entity:
        :param table_name:
        :param files_in_upload: a number of files in upload
        :param json_columns: list of columns need to beatify as json
        """
        UploaderDBBase.__init__(
            self,
            entity=entity,
            s3_client=s3_client,
            sql_credentials=sql_credentials,
            table_name=table_name,
            json_columns=json_columns,
            files_in_upload=files_in_upload,
            api_version=api_version,
        )

        self.logger = logging.getLogger(__class__.__name__)
        self.full_copy = full_copy
        self.db = "vertica"

    def load_s3_to_db(self):
        """the main method for running class"""
        with VerticaConnector(
                user=self.sql_credentials["user"],
                password=self.sql_credentials["password"],
                database=self.sql_credentials["database"],
                vertica_configs=self.sql_credentials["vertica_configs"],
                sec_to_recconect=2,
                count_retries=1,
        ) as v_connector:
            data = []
            cur_file = 1
            paths = self.s3_client.get_file_list(self.s3_client.root_dir)
            total_files = len(paths)
            try:
                self.columns = v_connector.get_columns(
                    table_name=self.table_name, schema=self.sql_credentials["schema"]
                )
            except ModuleNotFoundError as exc:
                self.logger.warning(exc)
                self.logger.error(
                    "You should create table %s.%s",
                    self.sql_credentials["schema"],
                    self.table_name,
                )
                if total_files > 0:
                    self.logger.info(
                        "You can try to create table like that %s",
                        self.generate_table_ddl(
                            paths[0], self.db, self.sql_credentials["schema"]
                        ),
                    )
                raise ModuleNotFoundError("Table not found exception") from exc

            if self.full_copy and len(paths) > 0:
                table_common_suffix = get_date_str(paths[0])
                v_connector.create_staging_table(table_name=self.table_name,
                                                 schema=self.sql_credentials["schema"],
                                                 staging_schema=self.sql_credentials["staging_schema"],
                                                 table_suffix=table_common_suffix
                                                 )

            for file_path in paths:
                self.logger.info("Loading data from the file %s", file_path)

                cur_data = self.process_json(
                    json.loads(self.s3_client.read_file(file_path)), self.entity
                )
                data += cur_data
                if cur_file % self.files_in_upload == 0 or cur_file == total_files:
                    data = self.update_data(
                        data=data, table_name=self.table_name, db=self.db
                    )
                    if not self.full_copy:
                        v_connector.insert_merge_vertica(
                            table_name=self.table_name,
                            schema=self.sql_credentials["schema"],
                            staging_schema=self.sql_credentials["staging_schema"],
                            data=data,
                            key_fields=[self.unique_column],
                            staging_table_suffix=get_date_str(file_path),
                        )
                    else:
                        v_connector.insert(
                            table_name=self.table_name,
                            schema=self.sql_credentials["staging_schema"],
                            data=data,
                            table_suffix=table_common_suffix
                        )
                    self.uploaded_rows += len(data)
                    self.log_status(cty_upload=len(data))
                    data = []
                cur_file += 1

            if self.full_copy:
                v_connector.reload_main_table(
                    table_name=self.table_name,
                    schema=self.sql_credentials["schema"],
                    staging_schema=self.sql_credentials["staging_schema"],
                    table_suffix=table_common_suffix
                )



"""
Module for downloading data from s3 to clickhouse
"""
import logging
import json
from abc import ABC

from clickhouse_balanced import Client
from amocrm.db.base.uploader import UploaderDBBase


class UploaderDB(UploaderDBBase, ABC):
    """
    Class to put the data from s3 into vertica
    """

    def __init__(
            self,
            s3_client,
            sql_credentials,
            entity,
            table_name,
            table_to_optimize=None,
            files_in_upload=5,
            full_copy=False,
            json_columns=None,
            api_version=4,
    ):
        """
        Constructor
        :param s3_client: s3 client from talenttech-oss library
        :param sql_credentials clickhouse variable dict
        :param entity:
        :param table_name:
        :param table_to_optimize: in a few cases
        :param files_in_upload: a number of files in upload
        :param full_copy: if True will make truncate and insert
        :param json_columns: list of columns need to beatify as json
        """
        UploaderDBBase.__init__(
            self,
            entity=entity,
            s3_client=s3_client,
            sql_credentials=sql_credentials,
            table_name=table_name,
            json_columns=json_columns,
            files_in_upload=files_in_upload,
            api_version=api_version,
        )

        self.logger = logging.getLogger(__class__.__name__)

        self.table_to_optimize = table_to_optimize
        self.full_copy = full_copy
        self.db = "ch"

    def __get_columns(self, table_name, ch_client):
        column_stat = """select name, type
            from  system.columns
            where table = '{table}'
                  and database = '{database}'
                  """.format(
            table=table_name, database=self.sql_credentials["database"]
        )
        rows = ch_client.execute(column_stat)
        res = dict(zip([row[0] for row in rows], [row[1] for row in rows]))
        return res

    def __delete_from_db(
            self, table_name, ch_client, to_delete_values=None, delete_column="id"
    ):
        """
        To delete already existed items from table
        Args:
            table_name:
            to_delete_values:  ids of deleted values
            delete_column: fields of ids

        Returns: Nothing
        """
        logging.info(
            "Deleting custom fields from %s, for %d leads",
            table_name,
            len(to_delete_values),
        )
        to_delete_values = to_delete_values or []
        columns = self.__get_columns(table_name, ch_client)
        columns_str = ",".join(columns)
        columns_changed_str = ",".join(
            ["-1" if c == "sign" else str(c) for c in columns]
        )

        if "sign" not in columns:
            logging.info("No sign for deleting in columns")
            return

        index = 0
        len_update = 1000
        row_deleted_count = len(to_delete_values)
        while index < len(to_delete_values):
            cur_deleted_values = ",".join(
                [str(r) for r in to_delete_values[index: index + len_update]]
            )
            select_st = (
                f"select {columns_changed_str} from {table_name} where sign = 1 "
                f"and {delete_column} in ({cur_deleted_values}) "
            )
            insert_st = f"insert into {table_name} ({columns_str}) {select_st} "
            ch_client.execute(insert_st)
            index += len_update

        logging.info(
            "Deleting custom fields from %s success, %d deleted leads",
            table_name,
            row_deleted_count,
        )

    def upload_data_to_db(self, data, ch_client, table_name=None):
        """
        Row uploading data to table
        Args:
            data:
            table_name:
        Returns: Nothing
        :param data:
        :param table_name:
        :param ch_client:
        """
        table_name = table_name or self.table_name
        columns = ",".join([c for c in self.columns if c in data[0].keys()])
        logging.info("Insert values to %s, a number is %d", table_name, len(data))
        logging.info("INSERT INTO %s (%s) VALUES", table_name, columns)
        ch_client.execute(
            f"INSERT INTO {table_name} ({columns}) VALUES", data, types_check=True
        )

    def __optimize_table_ch(self, table, ch_client, add=""):
        """optimize table, set only for clickhouse"""
        sql_cf_optimize = "Optimize table {database}.{table}  {add} ".format(
            database=self.sql_credentials["database"],
            table=table,
            add=add,
            cluster="{cluster}",
        )
        logging.info(sql_cf_optimize)
        ch_client.execute(sql_cf_optimize)

    def load_s3_to_db(self):
        """the main method for running class"""
        ch_client = Client(**self.sql_credentials)
        cur_file = 1
        data = []
        paths = self.s3_client.get_file_list(self.s3_client.root_dir)
        total_files = len(paths)
        self.columns = self.__get_columns(self.table_name, ch_client)
        if len(self.columns) == 0:
            self.logger.error(
                "You should create the table %s.%s",
                self.sql_credentials["database"],
                self.table_name,
            )
            if total_files > 0:
                self.logger.info(
                    "You can try to create the table like that %s",
                    self.generate_table_ddl(
                        paths[0], self.db, self.sql_credentials["database"]
                    ),
                )
            raise ModuleNotFoundError("Table not found exception")

        if self.full_copy:
            sql_truncate = f"truncate table {self.table_name}"
            logging.info(sql_truncate)
            ch_client.execute(sql_truncate)

        for file_path in paths:
            self.logger.info("Loading data from the file %s", file_path)

            cur_data = self.process_json(
                json.loads(self.s3_client.read_file(file_path)), self.entity
            )
            data += cur_data
            if cur_file % self.files_in_upload == 0 or cur_file == total_files:
                data = self.update_data(
                    data=data, table_name=self.table_name, db=self.db
                )
                self.upload_data_to_db(data=data, ch_client=ch_client)
                self.uploaded_rows += len(data)
                self.log_status(cty_upload=len(data))
                data = []
            cur_file += 1

        if self.table_to_optimize is not None:
            self.__optimize_table_ch(table=self.table_to_optimize, ch_client=ch_client, add=" final deduplicate")



from datetime import datetime, timedelta
import json
import requests


class AmoOAuthClient:
    """
    Client for authorization
    """

    def __init__(self, args_api, s3_client, logger, is_oauth2=True):
        """
        :param args_api:
        :param s3_client:
        :param is_oauth2:
        """
        self.args_api = args_api or {}
        self.is_oauth2 = is_oauth2
        self.s3_client = s3_client
        self.logger = logger
        self.__check_validity_args()
        if self.is_oauth2:
            self.access_token = None
        else:
            self.auth_cookie_str = None

    def __check_validity_args(self):
        """
        check if args api are valid
        :return:
        """
        if self.is_oauth2:
            must_have_arguments = ["CLIENT_SECRET", "CLIENT_ID", "REDIRECT_URL"]
        else:
            must_have_arguments = ["AMO_USER_LOGIN", "AMO_USER_HASH", "AMO_AUTH_URL"]

        for param in must_have_arguments:
            if param not in self.args_api:
                raise BaseException(
                    "Error. You have to specify {} param in args_api argument".format(
                        param
                    )
                )

    def __save_tokens_s3(self, data):
        """
        Save data with tokens to s3 / set only if is_oauth2=True
        :return:None
        """
        expires_in_datetime = datetime.now() + timedelta(seconds=data["expires_in"])
        data["expires_in_datetime"] = expires_in_datetime.strftime("%Y-%m-%d %H:%M")
        self.logger.info("Modifying config %s...", self.s3_client.secret_dir)
        self.s3_client.create_file(self.s3_client.secret_dir, json.dumps(data))
        self.logger.info(
            "Save new refresh and access token to %s... ", self.s3_client.secret_dir
        )

    def __get_tokens(self, args):
        """
        Get tokens and additional params
        :param args: dict with code or refresh_token
        :return: dict with refresh and access token
        """
        data = {
            "client_secret": self.args_api["CLIENT_SECRET"],
            "client_id": self.args_api["CLIENT_ID"],
            "redirect_uri": self.args_api["REDIRECT_URL"],
        }
        data.update(args)
        resp = requests.post(self.args_api["AUTH_URL"], data=data).json()
        if "refresh_token" not in resp and "access_token" not in resp:
            raise Exception(
                "An error occurred while retrieving auth params: " + str(resp)
            )
        return resp

    def oauth(self):
        """API authorization"""
        params = {
            "USER_LOGIN": self.args_api["AMO_USER_LOGIN"],
            "USER_HASH": self.args_api["AMO_USER_HASH"],
            "type": "json",
        }
        resp = requests.post(self.args_api["AMO_AUTH_URL"], data=params)
        response = resp.json()

        if response["response"]["auth"]:
            self.auth_cookie_str = resp.cookies
            self.logger.info(
                "AmoCRM: Authorized user %s ", self.args_api["AMO_AUTH_URL"]
            )
            return True

        self.logger.info(response["response"])
        raise ValueError("AmoCRM: Not authorized")

    def oauth2(self, code_auth=None):
        """API authorization auth2"""
        # read file from s3 if exists
        try:
            self.logger.info(
                "Load refresh and access token from file %s ", self.s3_client.secret_dir
            )
            response = json.loads(self.s3_client.read_file(self.s3_client.secret_dir))
            now_str = datetime.now().strftime("%Y-%m-%d %H:%M")
            if (
                    now_str < response["expires_in_datetime"]
            ):  # check if access token is still valid
                self.access_token = response["access_token"]
                return

            response = self.__get_tokens(
                args={
                    "refresh_token": response["refresh_token"],
                    "grant_type": "refresh_token",
                }
            )
        except Exception as exp:
            self.logger.warning(
                "If you got here you should regenerate refresh token"
            )
            if code_auth is None:
                raise NotImplementedError(
                    "You should pass code_auth argument to save the new refresh and access tokens "
                    " (or get and save  them the first time) "
                ) from exp
            response = self.__get_tokens(
                args={"code": code_auth, "grant_type": "authorization_code"}
            )

        self.access_token = response["access_token"]
        self.__save_tokens_s3(response)

    def auth(self, code_auth=None):
        """
        General authorization for both types
        :param code_auth:
        :return:
        """
        if self.is_oauth2:
            self.oauth2(code_auth)
        else:
            self.oauth()

    def get_headers(self):
        """
        :return: headers
        """
        if self.is_oauth2:
            return {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + self.access_token,
            }
        return {
            "Content-Type": "application/json",
        }

    def get_response_objects(self, url, params, headers=None):
        """
        Ger response by using current auth specification
        :param headers:
        :param url:
        :param params:
        :return: response
        """
        _headers = self.get_headers()
        if headers is not None:
            _headers.update(headers)
        if not self.is_oauth2:
            return requests.get(
                url, timeout=60, cookies=self.auth_cookie_str, headers=_headers, params=params
            )
        return requests.get(url, timeout=60, headers=_headers, params=params)



"""
Module for downloading data from s3 to vertica
"""

import logging
import sys
import json
import pandas as pd

from converter.fields_converter_oneway import FieldsConverterOneWay


def get_date_str(path):
    """get suffix to temporary table using path"""
    return (
        path.split("/")[-2]
        .replace(" ", "")
        .replace("-", "")
        .replace(":", "")
        .replace("+", "")
        .replace(".", "")
    )


class UploaderDBBase:
    """
    Class to put the data from s3 into vertica
    """

    def __init__(
        self,
        s3_client,
        sql_credentials,
        entity,
        table_name,
        files_in_upload=5,
        json_columns=None,
        api_version=4,
    ):
        """
        :param s3_client: s3 client from talenttech-oss library
        :param sql_credentials: vertica variable dict
        :param entity:
        :param table_name:
        :param files_in_upload: a number of files in upload
        :param json_columns: list of columns need to beatify as json
        """
        log_format = "%(asctime)-15s %(name)s:%(levelname)s: %(message)s"
        logging.basicConfig(format=log_format, stream=sys.stdout, level=logging.INFO)
        logging.basicConfig(format=log_format, stream=sys.stderr, level=logging.ERROR)
        logging.captureWarnings(True)
        self.logger = logging.getLogger(__class__.__name__)

        self.files_in_upload = files_in_upload
        self.sql_credentials = sql_credentials

        self.s3_client = s3_client

        self.api_version = api_version
        self.uploaded_rows = 0
        self.table_name = table_name
        self.json_columns = json_columns or []

        self.entity = entity
        self.columns = None
        self.unique_column = "id"

    def update_data(self, data, table_name, db):
        """
        DATE transformation for further update
        :param data:
        :param table_name:
        :param db: type of db
        :return: transformed data
        """
        converter = FieldsConverterOneWay(sql_credentials=None, db=db)
        df_original = pd.DataFrame(data=data)
        subset = None
        if "id" in df_original.columns:
            subset = ["id"]
        elif "uid" in df_original.columns:
            subset = ["uid"]
            self.unique_column = "uid"
        df_original = df_original.drop_duplicates(subset=subset)
        df_updated = df_original.where(pd.notnull(df_original), None).dropna(
            how="all", axis=1
        )
        for column in self.json_columns:
            if column in df_updated.columns:
                df_updated[column] = df_updated.apply(
                    lambda x: json.dumps(x[column], ensure_ascii=False)
                    if x[column] is not None
                    else None,
                    axis=1,
                )
        items = converter.update_value_type(
            table_name=table_name,
            items=df_updated.to_dict(orient="records"),
            fields=self.columns,
        )
        return items

    def process_json(self, data, entity):
        """
        :param data:
        :param entity:
        :return:
        """
        if self.api_version == 4:
            from amocrm.api.api_loader_amocrm_v4 import process_json

            return process_json(data, entity)
        elif self.api_version == 2:
            from amocrm.api.api_loader_amocrm_v2 import process_json

            return process_json(data, entity)

    def generate_table_ddl(self, file_path, db, schema):
        """
        Get approximate table ddl
        :param db:
        :param schema:
        :param file_path:
        :return:
        """
        converter = FieldsConverterOneWay(sql_credentials=None, db=db)
        cur_data = self.process_json(
            json.loads(self.s3_client.read_file(file_path)), self.entity
        )

        return converter.create_table_from_dataframe(
            dataframe=pd.DataFrame(data=cur_data),
            table_name=self.table_name,
            to_create=False,
            schema=schema,
        )

    def log_status(self, cty_upload):
        self.logger.info(
            "Loading %d rows to %s is successful,  a cumulative number is %d",
            cty_upload,
            self.table_name,
            self.uploaded_rows,
        )

    def load_s3_to_db(self):
        pass



"""class to read config from the file"""
import json
from operator import itemgetter

"""
 "projects": [
    {
      "config_name": "net",
      "entities": {
            "leads": {
                "extract": {
                    "amocrm_api_url": "https://netologyoutgoing.amocrm.ru/api/v4/leads?page={page}&limit={limit}",
                    "if_modified_since": "0",
                    "with_offset": "0"
                },
                "load": {
                    "vertiaca": {
                        "table_name": "amocrm_leads_v4"
                    },
                    "ch": {
                        "table_name": "amocrm_leads_v4",
                        "parser": [
                            {
                              "from_table": "amocrm_leads_v4",
                              "insert_table": "amocrm_leads_v4_custom_fields_values",
                              "script": ""
                            }
                          ]
                    }
                }
            }
      }      
"""


def get_config_list(config_dir):
    """
    Decompose config json to list of tuple (extract_params, [db_param1, db_param2])
    :param config_dir: path to config file
    :return: list of configs
    """
    result_list = []
    with open(config_dir) as file:
        config = json.load(file)
    projects = sorted(config["projects"], key=itemgetter("config_name"))
    for project in projects:
        entities = project["entities"]
        config_name = project["config_name"]

        for entity in entities:
            api_params = entities[entity]["extract"]
            api_params['config_name'] = config_name
            api_params['entity'] = entity
            api_params["action"] = "extract"
            db_type_params_list = []
            for db_type in entities[entity]["load"]:
                db_type_params = entities[entity]["load"][db_type]
                db_type_params['config_name'] = config_name
                db_type_params['entity'] = entity
                if 'parser' in db_type_params:
                    db_type_params['parser'] = json.dumps(db_type_params['parser'])
                db_type_params["action"] = "load"
                db_type_params["db"] = db_type
                db_type_params["if_modified_since"] = api_params["if_modified_since"]
                db_type_params_list.append(db_type_params)

            result_list.append((api_params, db_type_params_list))
    return result_list



"""
    CLASS LOADER from API TO S3, v2
    https://www.amocrm.com/developers/content/digital_pipeline/site_visit/
"""

import logging
from abc import ABC

from amocrm.api.base.api_loader_base import AmocrmApiLoaderBase

PAGE_NUMBER_MAX = 1e6


def process_json(data, entity):
    """
    update data after retrieving from s3
    Args:
        data: row data json
    Returns: list
    :param data:
    :param entity:
    """
    if entity in ("leads", "companies", "contacts", "tasks"):
        items = data["_embedded"]["items"]
    elif entity in "funnels":
        items = [item for key, item in data["_embedded"]["items"].items()]
    elif entity in "users":
        items = [item for key, item in data["_embedded"]["users"].items()]
    return items


class AmocrmApiLoader(AmocrmApiLoaderBase, ABC):
    """
    main class
    """

    def __init__(
        self,
        entity,
        s3_client,
        args_api,
        date_modified_from=None,
        batch_api=500,
        is_oauth2=True,
    ):
        """
        :param entity:   amocrm entities contacts/users/accounts e.t.c
        :param s3_client: s3 client from talenttech-oss library
        :param args_api: dict with AMO_USER_LOGIN/AMO_USER_HASH/AMO_AUTH_URL keys required
        :param date_modified_from: date update from where we are loading data
        :param batch_api: size of batch to upload
        """
        AmocrmApiLoaderBase.__init__(
            self,
            entity=entity,
            s3_client=s3_client,
            args_api=args_api,
            date_modified_from=date_modified_from,
            batch_api=batch_api,
            is_oauth2=is_oauth2,
        )

        self.logger = logging.getLogger(__class__.__name__)

    def extract(self):
        """load table from amocrm.api"""
        self.auth()
        self.create_s3_folder()
        self.clear_s3_folder()  # clear old before loading

        url_base = self.args_api["amocrm_api_url"]
        cur_offset = 0
        count_uploaded = self.batch_api

        params = {}
        headers = {}
        while cur_offset < PAGE_NUMBER_MAX and count_uploaded == self.batch_api:
            file_path = self.get_file_name(cur_offset, self.batch_api, 0)
            self.logger.info("Extracting page number %d ", cur_offset)
            if self.date_modified_from is not None:
                self.logger.info(
                    "Uploading data with after %s ", self.date_modified_from
                )
                url = url_base.format(batch_size_api=self.batch_api, offset=cur_offset)
                headers["IF-MODIFIED-SINCE"] = self.date_modified_from.strftime(
                    "%a, %d %b %Y %H:%M:%S UTC"
                )
            else:
                url = url_base
            self.logger.info(url)
            objects = self.oath_client.get_response_objects(url, params, headers)
            try:
                count_uploaded = len(process_json(objects.json(), self.entity))
                self.logger.info(
                    "Saving data to the file %s, count rows %d, total: %d",
                    file_path,
                    count_uploaded,
                    self.rows_to_upload,
                )
                self.s3_client.create_file(file_path, objects.content)
                self.rows_to_upload += count_uploaded
            except Exception as exc:
                self.logger.info("Can't load objects.content, exception: %s ", exc)
                self.logger.info("Objects.content: %s ", objects.content)
                return
            cur_offset += self.batch_api
        self.logger.info(
            "Total number of rows received from API is %d", self.rows_to_upload
        )
